@@ -1,12 +1,12 @@
 /* SBaz -- Scala Bazaar
- * Copyright 2005-2009 LAMP/EPFL
+ * Copyright 2005-2010 LAMP/EPFL
  * @author  Lex Spoon
  */
 
-// $Id: KeyRemoteKnown.scala 16895 2009-01-13 13:12:14Z cunei $
 
 package sbaz.clui.commands
 
+import sbaz.clui._
 import sbaz.keys._
 import sbaz.{messages => msg}
 
@@ -42,9 +42,9 @@ object KeyRemoteKnown extends Command {
           if(keys.isEmpty)
             Console.println("No known keys for " + chooseSimple.name)
           else {
-            val sortedKeys = keys.sort((a,b) => a.toString < b.toString)
+            val sortedKeys = keys.sortWith((a,b) => a.toString < b.toString)
             Console.println("Known keys for " + chooseSimple.name + ":")
-            for(val key <- sortedKeys)
+            for(key <- sortedKeys)
               Console.println("  " + key)
           }
         }
